import 'package:car_app/converter/point_latlng_converter.dart';
import 'package:car_app/exception/server.dart';
import 'package:car_app/model/point.dart';
import 'package:car_app/view/defaults.dart';
import 'package:car_app/view/routes_titles.dart';
import 'package:car_app/viewmodel/tracking.dart';
import 'package:drop_down_list/drop_down_list.dart';
import 'package:drop_down_list/model/selected_list_item.dart';
import 'package:flutter/material.dart';
import 'package:flutter_map/flutter_map.dart';
import 'package:flutter_map_location_marker/flutter_map_location_marker.dart';
import 'package:latlong2/latlong.dart';
import 'package:provider/provider.dart';

class TrackingView extends StatefulWidget {

  const TrackingView({super.key});

  @override
  State<StatefulWidget> createState() => _TrackingViewState();
}

class  _TrackingViewState extends State<TrackingView> {
  final List<LatLng> _points = [];
  final PointToLatLngConverter _pointLatLngConverter = PointToLatLngConverter(); // MVVM: should be in VM, but performance suffers a lot.
  late final AppLifecycleListener _lifecycleListener;
  final List<SelectedListItem> _teams = [];

  void _onLocationUpdate(Point point) {
    setState(() {
      _points.add(_pointLatLngConverter.convert(point));
    });
  }

  void _clearPoints() {
    setState(() {
      _points.clear();
    });
  }

  void _setTeams() {
    var viewModel = Provider.of<TrackingViewModel>(context, listen: false);
    viewModel.teamService.getTeams() // Done here instead of vm to eliminate the need to convert Teams to a List<SelectedListItem> on each state change.
      .forEach((team) {
        setState(() {
          _teams.add(SelectedListItem(name: team.teamName, value: team.teamId));
        });
      })
      .catchError(test: (e) => e is ServerException, (e, s) {
        viewModel.loggingService.error(e.toString(), exception: e, trace: s);
      }
    );
  }

  @override
  void initState() {
    super.initState();
    var viewModel = Provider.of<TrackingViewModel>(context, listen: false);
    _lifecycleListener = AppLifecycleListener(
      onInactive: viewModel.locationServiceToBackground,
      onResume: viewModel.locationServiceToForeground,
    );

    viewModel.authenticationService.isLoggedIn()
      .then((isLoggedIn) {
        if (isLoggedIn) {
          _setTeams();
        }
      });
  }

  @override
  void dispose() {
    _lifecycleListener.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: DefaultAppBar(
        context,
        titleText: TrackingRouting.title,
      ),
      drawer: DefaultDrawer(
        context,
        currentRoute: TrackingRouting.route,
      ),
      body: Consumer<TrackingViewModel>(
        builder: (_, viewModel, __) {
          return FlutterMap(
            options: MapOptions(
              initialCenter: const LatLng(58.9388, 5.6904),
              initialZoom: 15,
              backgroundColor: Theme.of(context).colorScheme.background,
            ),
            children: [
              TileLayer(
                urlTemplate: 'https://tile.openstreetmap.org/{z}/{x}/{y}.png',
                userAgentPackageName: "io.joos.car_app",
                tileBuilder: (context, tileWidget, tile) {
                  if(Theme.of(context).brightness == Brightness.dark) {
                    return ColorFiltered(
                      colorFilter: const ColorFilter.matrix(<double>[
                        -1,  0,  0, 0, 255,
                        0, -1,  0, 0, 255,
                        0,  0, -1, 0, 255,
                        0,  0,  0, 1,   0,
                      ]),
                      child: tileWidget,
                    );
                  }
                  return tileWidget;
                },
              ),
              PolylineLayer(
                polylines: [
                  Polyline(
                    points: _points,
                    color: Colors.red,
                    strokeWidth: 5,
                  ),
                ],
              ),
              CurrentLocationLayer(
                alignPositionOnUpdate: AlignOnUpdate.always,
              ),
              Center(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.end,
                  children: [
                    Container(
                      margin: const EdgeInsets.only(bottom: 20),
                      child: FloatingActionButton.extended(
                        onPressed:() => viewModel.toggleTracking(
                          _clearPoints,
                          _onLocationUpdate,
                          (String errorMessage) => showDialog(
                            context: context,
                            builder: (_) => AlertDialog(
                              title: const Text("Error"),
                              content: Text(errorMessage),
                              actions: [
                                TextButton(
                                  onPressed: () => Navigator.pop(context),
                                  child: const Text("Ok"),
                                )
                              ],
                            )
                          ),
                          () => showDialog<bool>(
                            context: context,
                            builder: (_) => AlertDialog(
                              title: const Text("Team not set"),
                              content: const Text("You didn't set a team. While tracking without a team is supported, the team cannot be changed later!"),
                              actions: [
                                TextButton(
                                  onPressed: () => Navigator.pop(context, false),
                                  child: const Text("Cancel"),
                                ),
                                TextButton(
                                  onPressed: () => Navigator.pop(context, true),
                                  child: const Text("Start anyway"),
                                ),
                              ],
                            )
                          )
                        ),
                        label: Text(viewModel.trackingButtonText),
                      ),
                    ),
                  ],
                ),
              ),
              Column(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  Align(
                    alignment: Alignment.bottomRight,
                    child: Container(
                      margin: const EdgeInsets.only(right: 10, bottom: 10),
                      child: ElevatedButton(
                        onPressed: () => {
                          DropDownState(
                            DropDown(
                              data: _teams,
                              enableMultipleSelection: false,
                              selectedItems: (List<SelectedListItem> selectedList) {
                                var selectedItem = selectedList[0]; // since enableMultipleSelection: false
                                viewModel.setTeamId(selectedItem.value!); // a value has been certainly set
                              },
                            ),
                          ).showModal(context)
                        },
                        style: ElevatedButton.styleFrom(
                          shape: const CircleBorder(),
                          padding: const EdgeInsets.all(2),
                        ),
                        child: const Icon(Icons.people_rounded)
                      ),
                    ),
                  ),
                ],
              ),
            ],
          );
        }
      )
    );
  }
}
